/// <reference types="node" />
export interface VersionBytes {
    bip32: {
        public: number;
        private: number;
    };
    public: number;
}
export declare const HDKEY_VERSIONS: {
    [key: string]: VersionBytes;
};
export interface HDKeyConstructorOptions {
    chainCode: Buffer;
    privateKey?: Buffer | null;
    publicKey?: Buffer | null;
    index?: number;
    depth?: number;
    parentFingerprint?: Buffer;
    version?: VersionBytes;
}
export declare class HDKey {
    private _version;
    private _privateKey?;
    private _publicKey;
    private _chainCode;
    private _index;
    private _depth;
    private _parentFingerprint?;
    private _keyIdentifier;
    constructor({ privateKey, publicKey, chainCode, index, depth, parentFingerprint, version }: HDKeyConstructorOptions);
    static parseMasterSeed(seed: Buffer, version?: VersionBytes): HDKey;
    static parseExtendedKey(key: string, version?: VersionBytes): HDKey;
    readonly privateKey: Buffer | null;
    readonly publicKey: Buffer;
    readonly chainCode: Buffer;
    readonly depth: number;
    readonly parentFingerprint: Buffer | null;
    readonly index: number;
    readonly keyIdentifier: Buffer;
    readonly fingerprint: Buffer;
    readonly version: VersionBytes;
    readonly extendedPrivateKey: string | null;
    readonly extendedPublicKey: string;
    derive(chain: string): HDKey;
    private deriveChildKey;
    private serialize;
}
